import DeepClone from '../../global/js/DeepClone';

const DEFAULT = {
    text: "是否确认",
    done() {},
    cancel() {}
};

class ConfirmWrapper {
    constructor(_el, _options) {
        this._options = Object.assign({}, DeepClone(DEFAULT), _options);
    }

    static getName() {
        return 'ConfirmWrapper';
    }

    static getScope() {
        return 'Global';
    }

    init() {
        let $el, $shadow, _this = this, text = _this._options.text;

        $shadow = $("<div class='cr-shadow'></div>").appendTo(document.body);
        $el = $(
            "<div class='cr-confirm-container'><div class='cr-confirm-content'>" + text + "</div>" +
            "<div class='cr-confirm-footer'><div class='cr-confirm-done'>是</div><div class='cr-confirm-cancel'>否</div></div></div>"
        ).appendTo($shadow);

        _this._el = $el;
        _this._shadow = $shadow;

        $el.on('click', '.cr-confirm-done', function() {
            _this.close();
            _this._options.done.call();
        });
        $el.on('click', '.cr-confirm-cancel', function() {
            _this.close();
            _this._options.cancel.call();
        });

        return _this;
    }

    open() {
        let $el = this._el;

        $el.addClass("show");
        return this;
    }

    close() {
        let $el = this._el, $shadow = this._shadow;

        $el.addClass("close");

        if("animation" in document.body.style) {
            $el.one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function() {
                $shadow.remove();
            });
        }
        else {
            // 兼容旧浏览器
            $shadow.remove();
        }

        return this;
    }
}

export default ConfirmWrapper;