class StepWrapper {
    constructor(_el, _options) {
        this._el = _el;
        this._options = _options;
    }
    static getName() {
        return "StepWrapper";
    }
    init() {
        const $el = $(this._el);
        const parts = this._options.parts;
        const total = parts.length;
        const widthNum = 12/total;
        const activeIndex = this._options.activeIndex || 0;
        const processValue = (100/total) * (activeIndex + 1);
        const $crArrow = $("<div class='row cr-arrow'></div>").insertAfter($el);
        const $crProcess = $("<div class='row cr-process'></div>").insertAfter($crArrow);

        this._program = $crProcess;
        this._arrow = $crArrow;
        this._activeIndex = activeIndex;
        this._stepWidth = 100/total;
        this._processValue = processValue;

        $el.addClass("row");

        for(let i = 0; i < total; i++) {
            let part = parts[i];
            let active = (activeIndex > i || activeIndex === i) ? "active" : "";
            $el.append(
                "<div class='cr-sign-container col-xs-"+ widthNum +" col-sm-"+ widthNum +" col-md-" + widthNum + " col-lg-" + widthNum + "'>" +
                "<div class='cr-step-sign " + active+ "'>" + part + "</div>" +
                "</div>"
            );
            $crArrow.append(
                "<div class='col-xs-"+ widthNum +" col-sm-"+ widthNum +" col-md-" + widthNum + " col-lg-" + widthNum + "'>" +
                "<div class='cr-arrow-top " + active + "'></div>" +
                "</div>"
            );

        }

        $crProcess.append(
            "<div class='progress'>" +
            "<div class='progress-bar progress-bar-striped active' role='progressbar' aria-valuenow='" +
            processValue + "' aria-valuemin='0' aria-valuemax='100' style='min-width: 2em;width: " + processValue + "%;'>" +
            "</div></div>"
        );

        return this;

    }
    nextStep() {
        const activeIndex = this._activeIndex + 1;
        stepFunc.call(this, activeIndex);
    }

    lastStep() {
        const activeIndex = this._activeIndex - 1;
        stepFunc.call(this, activeIndex);
    }
}

function stepFunc(activeIndex) {
    const $el = $(this._el);
    const $program = this._program;
    const $crArrow = this._arrow;
    const parts = this._options.parts;
    const total = parts.length;
    const processValue = (100/total) * (activeIndex + 1);

    if(processValue > 100 || activeIndex < 0) {
        return;
    }

    $crArrow.find('.cr-arrow-top').each((index, item) => {
        if(activeIndex > index || activeIndex === index) {
            $(item).addClass("active");
            $el.find('.cr-step-sign:eq(' + index + ')').addClass("active");
        }
        else {
            $(item).removeClass("active");
            $el.find('.cr-step-sign:eq(' + index + ')').removeClass("active");

        }
    });

    this._activeIndex = activeIndex;
    this._processValue = processValue;
    $program.attr('aria-valuenow', processValue);
    $program.find('.progress-bar').css('width', processValue + '%');
}

export default StepWrapper;