import '../../loader';
import '../../tip';
import '../../confirm';
import DeepClone from '../../global/js/DeepClone';
import 'jquery-placeholder';

const DEFAULT = {

    // 额外的头部工具按钮,格式遵循tools的格式,框架会为你额外添加到tools里面
    extraTools: null,

    // 默认有添加,修改,删除3个按钮, 如果您只需要添加,或者3个中的2个,请通过toolsFlag设置
    tools: [{
        text: "添加", icon: "glyphicon glyphicon-plus", handler: defaultInsert
    }, {
        text: "修改", icon: "glyphicon glyphicon-pencil", handler: defaultEdit
    }, {
        text: "删除", icon: "glyphicon glyphicon-minus", handler: defaultDelete
    }],

    // 设置添加,修改的时候弹出对话框的属性

    /*
        dialog: {
            id: ""// id的字段名,
            width: ""// 对话框的宽度,
            height: ""// 对话框的高度,
            path: ""// 对话框的iframe地址,
            deleteRemote: ""// 删除请求地址,
            isFull: false // 是否使用full 对话框, 默认是使用常规对话框
        }
     */

    dialog: null,

    // 行按钮
    /*
        [{
            text: "按钮", icon: "glyphicon glyphicon-cloud", color: "PRIMARY", handler: tools1
        }, {
            text: "按钮", icon: "glyphicon glyphicon-search", color: "DEFAULT", handler: tools2
        }]
     */

    /***
     * @Param
     *          text        String      按钮文本
     *          icon        String      按钮图标
     *          color       String      按钮颜色        PRIMARY, DEFAULT, WARNING, SUCCESS, DANGER, INFO
     *          handler     Function    点击事件        第一个参数是event, 第二个参数是这一行的数据
     *          condition   Function    出现的条件      condition不存在的时候,表示按钮始终出现,
     *                                                 condition存在的话, 只有condition返回true,按钮才出现
     *
     */
    rowTools: [],

    /***
     * 采用二进制移位算法,1=添加,2=修改,4=删除
     * 举个例子:
     *
     * 如果你设置的6,那么就只会显示修改删除2个按钮,因为2+4=6
     */
    toolsFlag: 7,

    // 是否允许搜索,这个选项关系到是否会显示出搜索栏
    enableSearch: true,

    // 这个选项可以设置搜索框内提示文字,不同的表格搜索条件是不同的
    searchPlaceholder: "输入搜索",

    // 远程地址,获取数据用的,这个选项优先于source选项
    sourceUrl: null,

    // 本地json数据
    source: null,

    // 点击搜索按钮的时候，走这个地址去检索
    /*
        这个值主要是为了解决如下的场景：
        第一次加载的时候是source本地数据，然后点击搜索的时候去搜索远程数据
        需求很变态，举个例子，流程中甲方是a和b，要显示个列表，第一次把a，b显示出来，点击搜索的时候去后端调取所有甲方的数据

        注意：
        这个值不要轻易去设置，除非你遇到了上述场景，因为它会去覆盖掉sourceUrl的值，并且会导致source设置无效
     */
    searchUrl: null,

    // 是否显示分页
    pagination: true,

    // 是否带有搜索记忆功能,存储前几次搜索选中的数据, 记忆的数据不参与分页
    memorable: false,

    /*
        不需要缓存, 针对memorable属性的记忆功能
        如果表格设置了memorable,但是有时候确实是不需要记忆功能
        比如dialog操作结束之后,需要自动reload,此时的reload不需要将选中的数据记忆下来
     */
    noCache: false,

    // @type Object 自定义远程传递参数
    remoteParam: null,

    /***
     * 分页参数设置
     *
     * @param pageSize : '每页显示条数'
     * @param curPage: '当前页数'
     */
    pager: {
        pageSize: 10,
        curPage: 1,
        total: 0,
        totalPage: 0
    },

    // 是否单选,默认是多选
    singleSelected: false,

    // 这个参数你不需要设置,框架里自己使用的,当然考虑到,如果你要设置的话,也是可以的

    /***
     * 这个选项是存储搜索关键字的
     * 一般不需要设置这个参数,而是框架用来存储用的
     *
     * 当然,如果设置了,那么框架会把你的设置作为额外的设置,一起进行判断
     *
     */
    searchFields: [],

    // 有时候列表需要在初始化的时候不加载数据,填入搜索条件的时候再加载,这个选项可以帮助你
    isEmptyOnInit: false,

    /***
     * 点击某一行事件, 特别注意:
     * 1. 单选的时候,取消选中不会触发这个事件
     * 2. 行按钮点击或者是某个单元格的点击事件不会触发这个事件
     *
     * @param event 点击事件参数
     * @param index 这一行的下标, 从0开始
     * @param row 本行数据对象
     * @param isSelect 是否选中, true : 选中, false : 取消选中
     */
    clickRowEvent: function(event, index, row, isSelect) {},

    /***
     * 选中某一行事件, 这个和上面的区别就是只有选中才会触发这个事件
     * 特别注意:
     * 1. 行按钮点击或者是某个单元格的点击事件不会触发这个事件
     *
     * @param event 点击事件参数
     * @param index 这一行的下标, 从0开始
     * @param row 本行数据对象
     */
    selectRowEvent: function(event, index, row) {},

    // columns 参数是十分重要的,这里没有给出默认值, 下面是对columns参数的说明

    /***
     *
     * field            String  字段名,与数据的key值对应, 当field == 'ck'的时候框架会解析成选择框
     * title            String  列名
     * isSearchField    Boolean 是否搜索关键字
     * cellClick        Function    单元格点击事件 value, row, event, field
     *
     *          @param   value 单元格的值
     *                   row 本行数据
     *                   event   点击事件对象参数
     *                   field   单元格的字段名
     *
     * formatter         Function    单元格格式化  value
     *
     *          @param  value   单元格的值
     *
     * sort              Boolean 服务端排序, DEFAULT false
     * dbField           String  数据库的字段名, 如果不设置这个值, 默认是使用field字段
     */

    /*
        例如如下配置:
        columns: [{
            field:'ck'
        }, {
            field:'name',title:'名称', formatter: function(value) {
                if(value === '2') {
                    return '1';
                }
                else {
                    return '3';
                }
            }
        }, {
            field:'age',title:'年龄'
        }, {
            field:'serialNO',title:'案件编号', cellClick: function(value, trData, event, field) {
            }
        }]
     */
};

function defaultInsert() {

    let $table = $(this).closest('.cr-table-container').find('.table'),
        _this = $table.data("self"), dialog = _this._options.dialog;

    if(!dialog) {
        $.TipWrapper({icon: "glyphicon glyphicon-bell", message: "请设置对话框属性"}).open();
        return;
    }

    let config = {
        width: dialog.width,
        height: dialog.height,
        title: "添加",
        content: "<iframe style='display: block' src='" + dialog.path + "'  width='100%' height='100%' frameborder='0'></iframe>"
    };

    if(dialog.isFull && "openFullDialog" in window) {
        window.openFullDialog(config);
    }
    else if(dialog.isFull && "openFullDialog" in top) {
        window.top.openFullDialog(config);
    }
    else if(!dialog.isFull && "openDialog" in window) {
        window.openDialog(config);
    }
    else if(!dialog.isFull && "openDialog" in top) {
        window.top.openDialog(config);
    }

}

function defaultEdit() {

    let $table = $(this).closest('.cr-table-container').find('.table'),
        _this = $table.data("self"), dialog = _this._options.dialog, row = _this.getSelectedRow();

    if(!dialog) {
        $.TipWrapper({icon: "glyphicon glyphicon-bell", message: "请设置对话框属性"}).open();
        return;
    }

    if (row === null) {
        $.TipWrapper({message: '请选择一条数据', icon: 'glyphicon glyphicon-bullhorn'}).open();
    }
    else {
        let path = dialog.path, icon = path.indexOf("?") > -1 ? "&" : "?";

        let config = {
            width: dialog.width,
            height: dialog.height,
            title: "修改",
            content: "<iframe style='display: block' src='" + path + icon + "objid=" + row[dialog.id] + "'  width='100%' height='100%' frameborder='0'></iframe>"
        };

        if(dialog.isFull && "openFullDialog" in window) {
            window.openFullDialog(config);
        }
        else if(dialog.isFull && "openFullDialog" in top) {
            window.top.openFullDialog(config);
        }
        else if(!dialog.isFull && "openDialog" in window) {
            window.openDialog(config);
        }
        else if(!dialog.isFull && "openDialog" in top) {
            window.top.openDialog(config);
        }
    }

}

function defaultDelete() {
    let $table = $(this).closest('.cr-table-container').find('.table'),
        _this = $table.data("self"), dialog = _this._options.dialog, row = _this.getSelectedRow();

    if(!dialog) {
        $.TipWrapper({icon: "glyphicon glyphicon-bell", message: "请设置对话框属性"}).open();
        return;
    }

    if (row === null) {
        $.TipWrapper({message: '请选择一条数据', icon: 'glyphicon glyphicon-bullhorn'}).open();
    }
    else {
        $.ConfirmWrapper({
            text: "是否删除",
            done() {
                $.post(dialog.deleteRemote, {'objid': row[dialog.id]}, function (data) {
                    if (data.success) {
                        _this.reload();
                        $.TipWrapper({message: '刪除成功', icon: 'glyphicon glyphicon-success'}).open();
                    } else {
                        $.TipWrapper({message: data.errorMsg, icon: 'glyphicon glyphicon-faild'}).open();
                    }
                });
            }
        }).open();
        // let confirm = window.confirm("是否删除");
        // if(confirm) {
        //     $.post(dialog.deleteRemote, {'objid': row[dialog.id]}, function (data) {
        //         if (data.success) {
        //             _this.reload();
        //             $.TipWrapper({message: '刪除成功', icon: 'glyphicon glyphicon-success'}).open();
        //         } else {
        //             $.TipWrapper({message: data.errorMsg, icon: 'glyphicon glyphicon-faild'}).open();
        //         }
        //     });
        // }
    }
}

/***
 * 搜索事件, 时间参数self不同情况下触发的话,会出现在不同的位置
 *
 * 如果是鼠标点击搜索按钮self参数在event.data._self里面
 * 如果是回车按钮快捷键触发的画,self参数在方法的第二个参数里面
 *
 * @param event
 * @param self
 */
function searchEvt(event, self) {
    let $search = $(this), $input = $search.closest('.input-group').children('.form-control'),
        _self = self || event.data._self;

    if ($input.val() && $input.val().trim() !== '') {
        _self._options.searchParam = {searchVal: $input.val().trim()};
    }
    // 如果没有搜索条件,显示所有数据
    else {
        _self._options.searchParam = null;
    }

    // 若需要带有搜索记忆功能,将选中的数据放入table.data("store");
    // if(_self._options.memorable === true) {
    //     let $el = $(_self._el);
    //     $el.data("store", _self.getCheckedRow());
    // }

    // 如果要重新设置远程访问地址
    if(_self._options.searchUrl !== null) {
        _self._options.sourceUrl = _self._options.searchUrl;
    }

    renderTable.call(_self);
}

function renderTools(tools) {

    let _this = this, $el = $(_this._el), options = _this._options;

    if ((!tools || tools.length === 0) && !options.enableSearch) return;

    let $toolsContainer = $("<div class='cr-table-tools row'></div><br/>").insertBefore($el);
    let $toolsBtnContainer =
        $("<div class='col-xs-7 col-sm-7 col-md-7 col-lg-7'></div>").appendTo($toolsContainer);

    // 如果存在工具
    if(tools && tools.length > 0) {
        let $toolsBtnGroup = $("<div class='btn-group btn-group-sm cr-table-button' role='group'></div>").appendTo($toolsBtnContainer);

        for (let i = 0, len = tools.length; i < len; i++) {
            let tool = tools[i];
            if(!tool) continue;

            let $tools = $("<button type='button' class='btn btn-default'><i class='" + tool.icon + "'></i> " + tool.text + "</button>").appendTo($toolsBtnGroup);
            $tools.on('click', tool.handler);
        }
    }

    // 如果允许搜索,添加搜索框
    if (options.enableSearch) {
        let $toolsSearchContainer =
            $("<div class='col-xs-5 col-sm-5 col-md-5 col-lg-5'></div>").appendTo($toolsContainer);
        $toolsSearchContainer.append(
            "<div class='input-group input-group-sm'>" +
            "<input class='form-control' placeholder='" + options.searchPlaceholder + "'>" +
            "<span class='input-group-btn'>" +
            "<button class='btn btn-primary searchBtn' type='button'>搜索</button>" +
            "</span></div></div>"
        );

        $toolsSearchContainer.find("input").placeholder();
        $toolsSearchContainer.on('click', '.searchBtn', {_self: _this}, searchEvt);

        // 输入结束之后添加回车搜索快捷键
        $toolsSearchContainer.on('focus', '.form-control', function() {
            $(document).on("keydown", function(event) {
                let evt = window.event || event, keyCode = evt.keyCode;
                if(keyCode === 13) {

                    // 这样写没办法定位到this
                    // searchEvt(event, _this);
                    // 这样写会触发两次,因为searchBtn元素带有before和after元素
                    // $(".searchBtn").trigger("click", _this);
                    // 这样写是最完美的
                    searchEvt.call($('.searchBtn')[0], event, _this);

                }
            });
        });

        // 点击外部取消回车快捷键
        $toolsSearchContainer.on('blur', '.form-control', function() {
            $(document).off("keydown");
        });

    }

    // 存储工具栏
    this._tools = $toolsContainer;
}

// 空数据处理
function resolveEmpty() {
    let _this = this, $el = $(_this._el), col = $el.find("th").length, columns = _this._options.columns;
    $el.find("tbody").append("<tr><td colspan='" + col + "' class='emptyData'>没有数据</td></tr>");

    // 如果没有数据,将不会进入columns循环,就不会存入isSearchField=true的列名,所以要在这边完成
    // 否则,输入搜索条件搜索的时候会出现意想不到的结果
    columns.map((column, i) => {
        let dbField = column.dbField ? column.dbField : column.field;
        if (column.isSearchField && this._options.searchFields.indexOf(dbField) < 0) {
            this._options.searchFields.push(dbField);
        }
    });

    // 关闭加载框
    setTimeout(function() {
        // isEmptyOnInit = true 的时候没有loader
        if(_this._loader) {
            _this._loader.close();
        }
    },500);
}

function renderTable() {
    let $el = $(this._el), options = this._options, _this = this;

    // 若需要带有搜索记忆功能,将选中的数据放入table.data("store");
    if(options.noCache === false && options.memorable === true) {
        $el.data("store", _this.getCheckedRow());
    }

    // 渲染表头
    renderHead.call(this);

    if(options.isEmptyOnInit) {
        options.isEmptyOnInit = false;
        resolveEmpty.call(this);
        return;
    }

    this._loader = $.LoaderWrapper().open();

    if (this._options.sourceUrl) {
        new Promise((resolve, reject) => {
            let param = {
                curPage: options.pager.curPage,
                pageSize: options.pager.pageSize,
                searchFields: options.searchFields
            };

            // 将需要必须传递的表格参数,搜索参数,自定义参数进行合并一起传递到后台
            param = Object.assign(param, options.searchParam, options.remoteParam);

            $.post(options.sourceUrl, param, function (data) {
                if (data.success) {
                    let result = data.data;
                    _this._options.data = result ? result.list : null;
                    _this._options.pager.total = result ? result.total : 0;
                    renderRows.call(_this);
                }
                else {
                    $.TipWrapper({message: "远程获取表格数据失败"}).open();
                }
            });
        });
    }
    else if (this._options.source) {
        let curPage = this._options.pager.curPage, pageSize = this._options.pager.pageSize,
            start = (curPage - 1) * pageSize,
            end = curPage * pageSize > this._options.source.length ? this._options.source.length : curPage * pageSize;

        this._options.pager.total = this._options.source.length;

        // 先深拷贝一份,存入data
        this._options.data = JSON.parse(JSON.stringify(this._options.source));

        /***
         * 如果存在过滤条件,先过滤,然后再分页,否则总页数和总条数是总数据的页数和条数
         *
         * 例如,数据[1,2,3],过滤掉3之后,总条数应该是2条
         * 如果先分页再过滤的画,总条数会显示成3条
         */

        // 如果存在搜索条件, 要进行过滤
        if (options.searchParam && options.searchFields.length > 0) {
            for (let i = this._options.data.length - 1; i >= 0; i--) {
                const row = this._options.data[i];
                options.searchFields.map((field, j) => {
                    // 当这一行数据的某个字段不存在搜索条件的时候,进行删除
                    if (row[field] && row[field].indexOf(options.searchParam.searchVal) < 0) {
                        this._options.data.splice(i, 1);
                    }
                });
            }
            this._options.pager.total = this._options.data.length;
        }

        // 如果开发者在使用的时候，传递了错误的数据给了table框架，就提示错误，并且停止渲染表格
        if(Object.prototype.toString.call(this._options.data) !== '[object Array]') {
            $.TipWrapper({message: "table框架接受的数据必须是个数组"}).open();
        }
        else {
            this._options.data = this._options.data.slice(start, end);

            renderRows.call(this);
        }

    }
    // 如果source和sourceUrl都是空的话,按空数据处理
    else {
        resolveEmpty.call(this);
    }

}

function renderRows() {
    let _this = this, $el = $(_this._el), columns = _this._options.columns,
        data = _this._options.data, storeCount, orderNum = 0;
    $el.find("tbody").empty();

    // 如果记忆中存在数据,将其与获取到的数据进行合并
    if($el.data("store")) {
        if(data && data.length > 0) {
            data = $el.data("store").concat(data);
        }
        else {
            data = $el.data("store");
        }
        storeCount = $el.data("store").length;
    }

    if(!data || data.length == 0) {
        resolveEmpty.call(this);
        return;
    }

    for (let i = 0, dlen = data ? data.length : 0; i < dlen; i++) {
        let $row = $("<tr></tr>").appendTo($el.find("tbody"));
        orderNum++;

        // list 可能会包含null值,直接跳过
        if(!data[i]) {
            continue;
        }
        for (let j = 0, clen = columns.length; j < clen; j++) {
            let column = columns[j], value = data[i][column.field], cellClickSign = column.cellClick ? "cellClick" : "";
            if (column.field === 'ck') {
                $row.append("<td><i class='glyphicon glyphicon-unchecked'></i></td>");
            }
            else if(column.field === "orderNum") {
                $row.append(`<td>${orderNum}</td>`);
            }
            else if(!column.hidden) {

                let $cellClick, $normalTD = $("<td></td>").appendTo($row);

                // 如果此列有格式化函数,执行格式化函数
                if(column.formatter) {
                    let _value = column.formatter.call(null, value),
                        reg = /<(?:(?:\/?[A-Za-z]\w*\b(?:[=\s](['"]?)[\s\S]*?\1)*)|(?:!--[\s\S]*?--))\/?>/g;//验证规则

                    $cellClick = $(
                        "<span title='" + (_value.match(reg) ? '' : _value) + "' class='" + cellClickSign + "'>" + _value + "</span>"
                    ).appendTo($normalTD);
                }
                // 如果没有, 直接显示数据
                else {
                    if(value === null || value === 'null') {
                        // $row.append("<td></td>");
                    }
                    else {
                        $cellClick = $(
                            "<span title='" + value + "' class='" + cellClickSign + "'>" + value + "</span>"
                        ).appendTo($normalTD);
                    }
                }

                // 绑定此单元格单击事件
                if($cellClick && column.cellClick) {
                    $cellClick.click(function(evt) {
                        let event = evt || window.event;
                        column.cellClick.call(null, value, data[i], event, column.field);
                    });
                }
            }

            // 如果是搜索关键字, 存入searchFields数组
            let dbField = column.dbField ? column.dbField : column.field;
            if (column.isSearchField && this._options.searchFields && this._options.searchFields.indexOf(dbField) < 0) {
                this._options.searchFields.push(dbField);
            }
        }

        // 如果存在行按钮则添加
        if(this._options.rowTools
            && this._options.rowTools.length > 0) {

            let $td = $("<td></td>").appendTo($row);

            this._options.rowTools.forEach((rowTool, index) => {
                let condition = rowTool.condition;

                // 不存在condition表示始终出现, 当condition存在的时候, 条件满足才会出现操作按钮
                if(!condition || condition.call(null, data[i])) {
                    let color = rowTool.color ? "label-" + rowTool.color.toLowerCase() : "label-primary";
                    let $label = $(
                        "<span class='label " + color + "'>" +
                        "<i class='" + rowTool.icon + "'></i>&nbsp;" + rowTool.text + "</span>"
                    ).appendTo($td);

                    index === rowTool.length || $td.append("&nbsp;");

                    if(rowTool.handler) {
                        $label.click(function(evt) {
                            let event = evt || window.event;
                            rowTool.handler.call(null, event, data[i]);
                        });
                    }
                }
            });
        }

        // 每条数据存入row的属性rowData中
        $row.data('rowData', data[i]);
        // 绑定行点击事件
        $row.on('click', {_self: this}, rowClk);
    }

    // 记忆的数据默认勾选状态
    if(storeCount) {
        for (var i = 0; i < storeCount; i++) {
            _this.activeRow(i);
        }
        $.TipWrapper({type: "warning", message: "记忆系统：存储了上次选中的" + storeCount + "条数据"}).open();
    }

    // 渲染分页
    if(this._options.pagination) {
        renderPage.call(this);
    }

    // 关闭加载框
    setTimeout(function() {
        _this._loader.close();
    },500);
}

function renderHead() {
    let $el = $(this._el);
    let columns = this._options.columns;
    $el.empty();

    let $head = $("<thead><tr></tr></thead>").appendTo($el);

    for (let i = 0, len = columns.length; i < len; i++) {
        let column = columns[i];
        if (column.field === 'ck') {

            if(this._options.singleSelected) {
                $head.children("tr").append("<th class='ck-columns'>状态</th>");
            }
            else {
                $head.children("tr").append("<th class='ck-columns ckAll'><i class='glyphicon glyphicon-unchecked'></i></th>");

                // 全选,取消全选事件
                $head.on('click', '.ckAll', function() {
                    let $ckAll = $(this), $row = $el.find("tbody tr");
                    // 还没有选中, 要设置全部选中
                    if($ckAll.children("i").hasClass("glyphicon-unchecked")) {
                        $row.find(".glyphicon-unchecked").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
                        $row.addClass("cr-row-checked");
                        $ckAll.children("i").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
                    }
                    // 已经全部选中, 要取消全部选中
                    else {
                        $row.find(".glyphicon-check").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
                        $row.removeClass("cr-row-checked");
                        $ckAll.children("i").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
                    }
                });
            }

        }
        else if(column.field === "orderNum") {
            $head.children("tr").append(`
                <th>序号</th>
            `);
        }
        else if(!column.hidden) {
            if(column.sort === true) {

                let sortDesc = column.sortDesc,
                    sortClass = sortDesc ?
                        sortDesc === "asc" ?
                            "glyphicon-sort-by-order" : "glyphicon-sort-by-order-alt" : "glyphicon-sort";
                // 添加服务端排序图表
                $head.children("tr").append(
                    "<th data-field='" + column.field + "'>" + column.title +
                    "&nbsp;&nbsp;<i class='glyphicon " + sortClass + " sort-icon'></i></th>"
                );
            }
            else {
                $head.children("tr").append("<th>" + column.title + "</th>");

            }
        }
    }

    // 如果存在行按钮,添加操作一列
    if(this._options.rowTools
        && this._options.rowTools.length > 0) {
        $head.children("tr").append("<th>操作</th>");
    }

    // 生序降序切换
    if($(".sort-icon").length > 0) {
        let _this = this;
        $el.off("click", ".sort-icon");
        $el.on("click", ".sort-icon", function() {
            let $sort_icon = $(this), $head = $sort_icon.closest("th"), order;

            // 常规切换到升序
            if($sort_icon.hasClass('glyphicon-sort')) {
                order = switchSort.call(
                    _this, $sort_icon, "glyphicon-sort", "glyphicon-sort-by-order", $head.data("field"), "asc"
                );
            }
            // 升序切换到降序
            else if($sort_icon.hasClass('glyphicon-sort-by-order')) {
                order = switchSort.call(
                    _this, $sort_icon, "glyphicon-sort-by-order", "glyphicon-sort-by-order-alt", $head.data("field"), "desc"
                );
            }
            // 降序切换到常规
            else if($sort_icon.hasClass('glyphicon-sort-by-order-alt')) {
                order = switchSort.call(
                    _this, $sort_icon, "glyphicon-sort-by-order-alt", "glyphicon-sort", $head.data("field"), null
                );
            }

            _this.reload({
                remoteParam: {
                    order: order
                }
            });
        });
    }

    $el.append("<tbody></tbody>");
}

function switchSort(
    $icon,
    from,
    to,
    field,
    order) {

    let orderStr;
    $icon.removeClass(from).addClass(to);
    order && (orderStr = field + " " + order);
    this._options.columns.forEach((column, index) => {
        if(column.field == field) {
            column.sortDesc = order;
        }
        else {
            column.sortDesc = null;
        }
    });

    return orderStr;

}

function rowClk(event) {
    let $row = $(this), _self = event.data._self, $target = $(event.target);

    // 行按钮和列单击按钮点击,不触发行点击事件
    if($target.hasClass('cellClick') || $target.hasClass('label')) {
        return false;
    }

    // 点击之前是选中状态, 点击之后要取消选中状态
    if ($row.hasClass('cr-row-checked')) {

        // 多选才触发行点击事件
        if (!_self._options.singleSelected) {
            let index = $(this).closest("table").find("tbody tr").index($row);
            _self._options.clickRowEvent.call(null, event, index, $row.data('rowData'), false);

            // 如果全选框是全选状态, 全选框要取消全选
            let $ckAll = $row.closest("tbody").prev("thead").find(".ckAll"),
                $i = $ckAll.children("i");
            if($i.hasClass("glyphicon-check")) {
                $i.removeClass("glyphicon-check").addClass("glyphicon-unchecked");
            }
        }
        $row.find(".glyphicon-check").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
    }
    // 点击之前不是选中状态, 点击之后要添加选中
    else {
        // 如果是单选去掉其他已经选中的行
        if (_self._options.singleSelected) {
            $row.siblings(".cr-row-checked").each((index, item) => {
                $(item).find(".glyphicon-check").removeClass("glyphicon-check").addClass("glyphicon-unchecked");
            }).removeClass('cr-row-checked');
        }
        else {
            let ckAllFlag = true;
            // 如果其他的tr都是选中状态, 全选框要设置选中
            $row.siblings("tr").each((index, item) => {
                if(!$(item).hasClass('cr-row-checked')) {
                    ckAllFlag = false;
                    return false;
                }
            });
            if(ckAllFlag === true) {
                $row.closest("tbody").prev("thead").find(".ckAll").children("i").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
            }
        }

        let index = $(this).closest("table").find("tbody tr").index($row);

        _self._options.clickRowEvent.call(null, event, index, $row.data('rowData'), true);
        _self._options.selectRowEvent.call(null, event, index, $row.data('rowData'));

        $row.find(".glyphicon-unchecked").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
    }

    $row.toggleClass("cr-row-checked");

}

function toHomePage() {
    let $table = $(this).closest('.cr-table-container').find('.table'), _this = $table.data("self");

    _this._options.pager.curPage = 1;
    renderTable.call(_this);
    $(this).closest('.cr-table-pagination').find('.curPage').val(_this._options.pager.curPage);
}

function toPreviousPage() {
    let $table = $(this).closest('.cr-table-container').find('.table'), _this = $table.data("self"),
        curPage = _this._options.pager.curPage;

    if (curPage == 1) return false;
    _this._options.pager.curPage--;
    renderTable.call(_this);
    $(this).closest('.cr-table-pagination').find('.curPage').val(_this._options.pager.curPage);
}

function toNextPage() {
    let $table = $(this).closest('.cr-table-container').find('.table'), _this = $table.data("self"),
        curPage = _this._options.pager.curPage, totalPage = _this._options.pager.totalPage;

    if (curPage == totalPage) return false;
    _this._options.pager.curPage++;
    renderTable.call(_this);
    $(this).closest('.cr-table-pagination').find('.curPage').val(_this._options.pager.curPage);
}

function toLastPage() {
    let $table = $(this).closest('.cr-table-container').find('.table'), _this = $table.data("self"),
        totalPage = _this._options.pager.totalPage;
    _this._options.pager.curPage = totalPage;
    renderTable.call(_this);
    $(this).closest('.cr-table-pagination').find('.curPage').val(_this._options.pager.curPage);
}

function toRefresh() {
    let $table = $(this).closest('.cr-table-container').find('.table'), _this = $table.data("self"),
        curPage = $(this).closest('.cr-table-pagination').find('.curPage').val(),
        pageSize = $(this).closest('.cr-table-pagination').find('.pageSize').val(),
        totalPage = _this._options.pager.totalPage;

    if (!(/(^[1-9]\d*$)/.test(curPage))) {
        $.TipWrapper({message: "当前页输入非法字符"}).open();
        return false;
    }
    if (!(/(^[1-9]\d*$)/.test(pageSize))) {
        $.TipWrapper({message: "每页显示输入非法字符"}).open();
        return false;
    }
    if (curPage > totalPage) {
        curPage = totalPage;
        $(this).closest('.cr-table-pagination').find('.curPage').val(curPage);
    }
    _this._options.pager.curPage = curPage;
    _this._options.pager.pageSize = pageSize;
    renderTable.call(_this);
}

function renderPage() {
    let $el = $(this._el), options = this._options, curPage = options.pager.curPage,
        pageSize = options.pager.pageSize, total = options.pager.total || 0,
        totalPage = this._options.pager.totalPage = total % pageSize > 0 ? parseInt(total / pageSize) + 1 : total / pageSize;

    // 删除再添加的话,发现这边display样式会丢, 所以索性直接复制一遍
    let display = $el.next('.cr-table-pagination').css("display");
    $el.next('.cr-table-pagination').remove();
    let $pageContainer = $("<div class='cr-table-pagination row'></div>").insertAfter($el);
    $pageContainer.css("display", display);
    this._pager = $pageContainer;

    $pageContainer.append(
        "<div class='col-lg-4 col-sm-4 col-xs-4 col-md-4 cr-pagination-container' >" +
        "<ul class='pagination pagination-sm'>" +
        "<li><a href='#' class='homePageBtn'><i class='glyphicon glyphicon-fast-backward'></i> 首页</a></li>" +
        "<li><a href='#' class='previousPageBtn'>上一页</a></li>" +
        "<li><a href='#' class='nextPageBtn'>下一页</a></li>" +
        "<li><a href='#' class='lastPageBtn'><i class='glyphicon glyphicon-fast-forward'></i> 末页</a></li>" +
        "<li><a href='#' class='refreshBtn'><i class='glyphicon glyphicon-refresh'></i> 刷新</a></li>" +
        "</ul></div>" +
        "<div class='col-lg-8 col-sm-8 col-xs-8 col-md-8'>" +
        "<div class='cr-page-size'>每页显示<input class='pageSize' value='" + pageSize + "'></input>条, 当前: " +
        "<input class='curPage' value='" + curPage + "'></input>页, 共<span class='total'>" + total + "</span>条, " +
        "<span class='totalPage'>" + totalPage + "</span>页</div></div>"
    );

    $pageContainer.find(".homePageBtn").on("click", toHomePage);
    $pageContainer.find(".previousPageBtn").on("click", toPreviousPage);
    $pageContainer.find(".nextPageBtn").on("click", toNextPage);
    $pageContainer.find(".lastPageBtn").on("click", toLastPage);
    $pageContainer.find(".refreshBtn").on("click", toRefresh);

}

class TableWrapper {

    constructor(_el, options) {
        this._el = _el;

        // let copy = JSON.parse(JSON.stringify(DEFAULT)), toolsCopy = DEFAULT.tools, rowToolsCopy = DEFAULT.rowTools;
        // copy.tools = toolsCopy, copy.rowTools = rowToolsCopy;
        this._options = Object.assign({}, DeepClone(DEFAULT), options);

        // 默认设置当前页为第一页, 如果没有设置这个,外部不存在curPage属性的话,
        // Object.assign是直接拷贝外部不存在curPage属性的pager对象,导致curPage不存在
        this._options.pager.curPage || (this._options.pager.curPage = 1);

        let toolsFlag = this._options.toolsFlag, ADD = 1 << 0, EDIT = 1 << 1, REMOVE = 1 << 2;
        (toolsFlag & ADD) === 0 && delete this._options.tools[0];
        (toolsFlag & EDIT) === 0 && delete this._options.tools[1];
        (toolsFlag & REMOVE) === 0 && delete this._options.tools[2];

    }

    static getName() {
        return "TableWrapper";
    }

    static getName2() {
        return "Table2Wrapper";
    }

    init() {
        const $el = $(this._el);

        $el.data("self", this);
        $el.addClass("table table-bordered table-hover");
        $el.wrap("<div class='cr-table-container'></div>");
        const settings = this._options;

        settings.tools = settings.extraTools ? settings.tools.concat(settings.extraTools) : settings.tools;
        renderTools.call(this, settings.tools);
        renderTable.call(this);

        return this;
    }

    reload(options) {

        // reload的时候可以对table的一些参数进行重新设置
        Object.assign(this._options, options);

        // 刷新的时候如果已经存在分页,直接出发分页按钮的点击事件即可
        if(this._pager) {
            this._pager.find(".refreshBtn").trigger('click');
        }
        // 刷新的时候如果还不存在分页,当没有数据的时候是没有分页的,
        // 添加完第一条数据要刷新的时候,分页还不存在,此时直接出发渲染表格方法
        else {
            renderTable.call(this);
        }
        return this;
    }

    // 返回选中的一行数据,如果选中多行,那么只会返回选中的第一行数据
    getSelectedRow() {
        let $el = $(this._el), $row = $el.find('.cr-row-checked:eq(0)');
        if ($row.length > 0) {
            return $row.data('rowData');
        }
        return null;
    }

    // 返回选中的多行数据
    getCheckedRow() {
        let $el = $(this._el), $rows = $el.find('.cr-row-checked'), result = [];
        if ($rows.length > 0) {
            $rows.each((index, item) => {
                result.push($(item).data('rowData'));
            });
            return result;
        }
        return null;
    }

    // 触发搜索事件
    triggerSearch(text) {
        let $toolsContrainer = this._tools, $input = $toolsContrainer.find("input"),
            $searchBtn = $toolsContrainer.find(".searchBtn");

        if($toolsContrainer) {
            $input.val(text);
            $searchBtn.trigger('click');
        }
    }

    activeRow(i) {
        let $el = $(this._el), $rows = $el.find('tbody tr');

        $rows.each((index, item) => {
            if(i === index) {
                $(item).find(".glyphicon-unchecked").removeClass("glyphicon-unchecked").addClass("glyphicon-check");
                $(item).toggleClass("cr-row-checked");
                return false;
            }
        })
    }
}

export default TableWrapper;